//
//  QuDownloadController.m
//  QuDownloader
//
//  Created by Alan Quatermain on 19/04/07.
//  Copyright 2007 AwkwardTV. All rights reserved.
//
// Updated by nito 08-20-08 - works in 2.x

#import "QuDownloadController.h"
#import "QuProgressBarControl.h"
#import "BRLocalizedString.h"
#import <BackRow/BackRow.h>
#define myDomain			(CFStringRef)@"org.quatermain.downloader"

static NSString * const kDefaultURLString = @"http://homepage.mac.com/jimdovey/nukes/nukes/files/BiggerThanYouEP.mp3";

@implementation QuDownloadController

+ (void) clearAllDownloadCaches
{
    [[NSFileManager defaultManager] removeFileAtPath: [self downloadCachePath]
                                             handler: nil];
}

+ (NSString *) downloadCachePath
{
    static NSString * __cachePath = nil;

    if ( __cachePath == nil )
    {
        // find the user's Caches folder
        NSArray * list = NSSearchPathForDirectoriesInDomains( NSCachesDirectory,
            NSUserDomainMask, YES );

        // handle any failures in that API
        if ( (list != nil) && ([list count] != 0) )
            __cachePath = [list objectAtIndex: 0];
        else
            __cachePath = NSTemporaryDirectory( );

        __cachePath = [[__cachePath stringByAppendingPathComponent: @"QuDownloads"] retain];

        // ensure this exists
        [[NSFileManager defaultManager] createDirectoryAtPath: __cachePath
                                                   attributes: nil];
    }

    return ( __cachePath );
}

+ (NSString *) outputPathForURLString: (NSString *) urlstr
{
    NSString * cache = [self downloadCachePath];
    NSString * name = [urlstr lastPathComponent];

    // trim any parameters from the URL
    NSRange range = [name rangeOfString: @"?"];
    if ( range.location != NSNotFound )
        name = [name substringToIndex: range.location];

    NSString * folder = [[name stringByDeletingPathExtension]
                         stringByAppendingPathExtension: @"download"];

    return ( [NSString pathWithComponents: [NSArray arrayWithObjects: cache,
        folder, name, nil]] );
}

- (void) drawSelf

{
	//ditto above comment where urlstr is commented out.
	NSString *urlstr;
	//NSString * urlstr = [[RUIPreferenceManager sharedPreferences] stringForKey: @"QuDownloadURL"
																//	 forDomain: @"org.quatermain.downloader"];
		
	
	//if ( urlstr == nil )
		urlstr = kDefaultURLString;
	
	_header = [[BRHeaderControl alloc] init];
	_sourceText = [[BRTextControl alloc] init];
	_progressBar = [[QuProgressBarControl alloc] init];
	
	// work out our desired output path
	_outputPath = [[QuDownloadController outputPathForURLString: urlstr] retain];
	
	// lay out our UI
	NSRect masterFrame = [self frame];
	NSRect frame = masterFrame;
	
	// header goes in a specific location
	frame.origin.y = frame.size.height * 0.82f;
	frame.size.height = [[BRThemeInfo sharedTheme] listIconHeight];
	[_header setFrame: frame];
	
	// progress bar goes in a specific place too (one-eighth of the way
	// up the screen)
	frame.size.width = masterFrame.size.width * 0.45f;
	frame.size.height = ceilf( frame.size.width * 0.068f );
	frame.origin.x = (masterFrame.size.width - frame.size.width) * 0.5f;
	frame.origin.y = masterFrame.origin.y + (masterFrame.size.height * (1.0f / 8.0f));
	[_progressBar setFrame: frame];
	
	[self setTitle: @"Download Test"];
	[self setSourceText: urlstr];   // this lays itself out
	[_progressBar setCurrentValue: [_progressBar minValue]];
	
	// add the controls
	[self addControl: _header];
	[self addControl: _sourceText];
	[self addControl: _progressBar];
	
	
}

- (id) init {
    if ( [super init] == nil )
        return ( nil );
	
	//this used to be initWithScene etc... RUIPreferenceManager is not a good idea to use in 2.x because
	//it is phased out in 2.1, this is commented out because the screen is not drawn until controlWillActivate, from there we call -(void)drawSelf
//
	//in lieu of RUIPreferenceManager being removed you can always use CoreFoundation
	//CFStringRef urlstr = (CFStringRef)CFPreferencesCopyAppValue((CFStringRef)@"QuDownloadURL", myDomain);
	
	
//    NSString * urlstr = [[RUIPreferenceManager sharedPreferences] stringForKey: @"QuDownloadURL"
//                         forDomain: @"org.quatermain.downloader"];
//
	
//    // if no URL, let's listen to my mate's band !
//    if ( urlstr == nil )
//        urlstr = kDefaultURLString;
//
//    _header = [[BRHeaderControl alloc] init];
//    _sourceText = [[BRTextControl alloc] init];
//    _progressBar = [[QuProgressBarControl alloc] init];
//
//    // work out our desired output path
//    _outputPath = [[QuDownloadController outputPathForURLString: urlstr] retain];
//
//    // lay out our UI
//    NSRect masterFrame = [[self masterLayer] frame];
//    NSRect frame = masterFrame;
//
//    // header goes in a specific location
//    frame.origin.y = frame.size.height * 0.82f;
//    frame.size.height = [[BRThemeInfo sharedTheme] listIconHeight];
//    [_header setFrame: frame];
//
//    // progress bar goes in a specific place too (one-eighth of the way
//    // up the screen)
//    frame.size.width = masterFrame.size.width * 0.45f;
//    frame.size.height = ceilf( frame.size.width * 0.068f );
//    frame.origin.x = (masterFrame.size.width - frame.size.width) * 0.5f;
//    frame.origin.y = masterFrame.origin.y + (masterFrame.size.height * (1.0f / 8.0f));
//    [_progressBar setFrame: frame];
//
//    [self setTitle: BRLocalizedString(@"QuDownloader Example", @"Title text")];
//    [self setSourceText: urlstr];   // this lays itself out
//    [_progressBar setCurrentValue: [_progressBar minValue]];
//
//    // add the controls
//    [self addControl: _header];
//    [self addControl: _sourceText];
//    [self addControl: _progressBar];

    return ( self );
}

- (void) dealloc
{
    [self cancelDownload];

    [_header release];
    [_sourceText release];
    [_progressBar release];
    [_downloader release];
    [_outputPath release];

    [super dealloc];
}

- (BOOL) beginDownload
{
    if ( _downloader != nil )
        return ( NO );

    // see if we can resume from the current data
    if ( [self resumeDownload] == YES )
        return ( YES );

    // didn't work, delete & try again
    [self deleteDownload];

	//this is commented out because RUIPreferenceManager will break support in 2.1
	NSString *urlstr = (NSString *)CFPreferencesCopyAppValue((CFStringRef)@"QuDownloadURL", myDomain);
   // NSString * urlstr = [[RUIPreferenceManager sharedPreferences] stringForKey: @"QuDownloadURL"
     //forDomain: @"org.quatermain.downloader"];

    //if ( urlstr == nil )
        urlstr = kDefaultURLString;

    NSURL * url = [NSURL URLWithString: urlstr];
    if ( url == nil )
        return ( NO );

    NSURLRequest * req = [NSURLRequest requestWithURL: url
                                          cachePolicy: NSURLRequestUseProtocolCachePolicy
                                      timeoutInterval: 20.0];

    // create the dowloader
    _downloader = [[NSURLDownload alloc] initWithRequest: req delegate: self];
    if ( _downloader == nil )
        return ( NO );

    [_downloader setDeletesFileUponFailure: NO];

    return ( YES );
}

- (BOOL) resumeDownload
{
    if ( _outputPath == nil )
        return ( NO );

    NSString * resumeDataPath = [[_outputPath stringByDeletingLastPathComponent]
                                 stringByAppendingPathComponent: @"ResumeData"];
    if ( [[NSFileManager defaultManager] fileExistsAtPath: resumeDataPath] == NO )
        return ( NO );

    NSData * resumeData = [NSData dataWithContentsOfFile: resumeDataPath];
    if ( (resumeData == nil) || ([resumeData length] == 0) )
        return ( NO );

    // try to initialize using the saved data...
    _downloader = [[NSURLDownload alloc] initWithResumeData: resumeData
                                                   delegate: self
                                                       path: _outputPath];
    if ( _downloader == nil )
        return ( NO );

    [_downloader setDeletesFileUponFailure: NO];

    return ( YES );
}

- (void) cancelDownload
{
    [_downloader cancel];
    [self storeResumeData];
}

- (void) deleteDownload
{
    if ( _outputPath == nil )
        return;

    [[NSFileManager defaultManager] removeFileAtPath:
        [_outputPath stringByDeletingLastPathComponent]
                                             handler: nil];
}

// stack callbacks
- (void)controlWasActivated;
{
	[self drawSelf];
	
    if ( [self beginDownload] == NO )
    {
        [self setTitle: @"Download Failed"];
        [_progressBar setPercentage: 0.0f];
        ////[[self scene] renderScene];
    }
	
    [super controlWasActivated];
}

- (void)controlWillDeactivate;
{
    [self cancelDownload];
    [super controlWillDeactivate];
}

- (BOOL) isNetworkDependent
{
    return ( YES );
}

- (void) setTitle: (NSString *) title
{
    [_header setTitle: title];
}

- (NSString *) title
{
    return ( [_header title] );
}
- (void) setSourceText: (NSString *) srcText
{
	//   [_sourceText setTextAttributes: [[BRThemeInfo sharedTheme] paragraphTextAttributes]];
    [_sourceText setText: srcText withAttributes:[[BRThemeInfo sharedTheme] paragraphTextAttributes]];
	
    // layout this item
    NSRect masterFrame = [self frame];
	
	// [_sourceText setMaximumSize: NSMakeSize(masterFrame.size.width * 0.66f,
	//                                       masterFrame.size.height)];
	
    CGSize txtSize = [_sourceText renderedSize];
	
    CGRect frame;
    frame.origin.x = (masterFrame.size.width - txtSize.width) * 0.5f;
    frame.origin.y = (masterFrame.size.height * 0.75f) - txtSize.height;
    frame.size = txtSize;
    
	[_sourceText setFrame: frame];
}

- (NSString *) sourceText
{
    return ( [_sourceText text] );
}

- (float) percentDownloaded
{
    return ( [_progressBar percentage] );
}

- (void) storeResumeData
{
    NSData * data = [_downloader resumeData];
    if ( data != nil )
    {
            // store this in the .download folder
        NSString * path = [[_outputPath stringByDeletingLastPathComponent]
                           stringByAppendingPathComponent: @"ResumeData"];
        [data writeToFile: path atomically: YES];
    }
}

// NSURLDownload delegate methods
- (void) download: (NSURLDownload *) download
   decideDestinationWithSuggestedFilename: (NSString *) filename
{
    // we'll ignore the given filename and use our own
    // they'll likely be the same, anyway

    // ensure that all new path components exist
    [[NSFileManager defaultManager] createDirectoryAtPath: [_outputPath stringByDeletingLastPathComponent]
                                               attributes: nil];

    NSLog( @"Starting download to file '%@'", _outputPath );

    [download setDestination: _outputPath allowOverwrite: YES];
}

- (void) download: (NSURLDownload *) download didFailWithError: (NSError *) error
{
    [self storeResumeData];

    NSLog( @"Download encountered error '%d' (%@)", [error code],
           [error localizedDescription] );

    // show an alert for the returned error (hopefully it has nice
    // localized reasons & such...)
    BRAlertController * obj = [BRAlertController alertForError: error
                                                     withScene: [self scene]];
    [[self stack] swapController: obj];
}

- (void) download: (NSURLDownload *) download didReceiveDataOfLength: (unsigned) length
{
    _gotLength += (long long) length;
    float percentage = 0.0f;

    //NSLog( @"Got %u bytes, %lld total", length, _gotLength );

    // we'll handle the case where the NSURLResponse didn't include the
    // size of the source file
    if ( _totalLength == 0 )
    {
        // bump up the max value a bit
        percentage = [_progressBar percentage];
        if ( percentage >= 95.0f )
            [_progressBar setMaxValue: [_progressBar maxValue] + (float) (length << 3)];
    }

    [_progressBar setCurrentValue: _gotLength];
}

- (void) download: (NSURLDownload *) download didReceiveResponse: (NSURLResponse *) response
{
    // we might receive more than one of these (if we get redirects,
    // for example)
    _totalLength = 0;
    _gotLength = 0;

    NSLog( @"Got response for new download, length = %lld", [response expectedContentLength] );

    if ( [response expectedContentLength] != NSURLResponseUnknownLength )
    {
        _totalLength = [response expectedContentLength];
        [_progressBar setMaxValue: (float) _totalLength];
    }
    else
    {
        // an arbitrary number -- one megabyte
        [_progressBar setMaxValue: 1024.0f * 1024.0f];
    }
}

- (BOOL) download: (NSURLDownload *) download
   shouldDecodeSourceDataOfMIMEType: (NSString *) encodingType
{
    NSLog( @"Asked to decode data of MIME type '%@'", encodingType );

    // we'll allow decoding only if it won't interfere with resumption
    if ( [encodingType isEqualToString: @"application/gzip"] )
        return ( NO );

    return ( YES );
}

- (void) download: (NSURLDownload *) download
   willResumeWithResponse: (NSURLResponse *) response
                 fromByte: (long long) startingByte
{
    // resuming now, so pretty much as above, except we have a starting
    // value to set on the progress bar
    _totalLength = 0;
    _gotLength = (long long) startingByte;

    // the total here seems to be the amount *remaining*, not the
    // complete total

    NSLog( @"Resumed download at byte %lld, remaining is %lld",
           _gotLength, [response expectedContentLength] );

    if ( [response expectedContentLength] != NSURLResponseUnknownLength )
    {
        _totalLength = _gotLength + [response expectedContentLength];
        [_progressBar setMaxValue: (float) _totalLength];
    }
    else
    {
        // an arbitrary number
        [_progressBar setMaxValue: (float) (_gotLength << 1)];
    }

    // reset current value as appropriate
    [_progressBar setCurrentValue: (float) _gotLength];
}

- (void) downloadDidFinish: (NSURLDownload *) download
{
    // completed the download: set progress full (just in case) and
    // go do something with the data
    [_progressBar setPercentage: 100.0f];

    NSLog( @"Download finished" );

    // we'll swap ourselves off the stack here, so let's remove our
    // reference to the downloader, just in case calling -cancel now
    // might cause a problem
    [_downloader autorelease];
    _downloader = nil;

    // By default I'm downloading a music file, so let's use a music
    // player for it, eh?
    NSError * error = nil;
    NSURL * url = [NSURL fileURLWithPath: _outputPath];
	
	/* this code below is from the original example and no longer works on 2.0 appletvs 
	
    BRBaseMediaAsset * asset = [[[BRSimpleMediaAsset alloc] initWithMediaURL: url] autorelease];

    BRMediaPlayer * player = [[[BRQTKitVideoPlayer alloc] init] autorelease];
    [player setMedia: asset error: &error];

    id controller = [[BRVideoPlayerController alloc] init];
    [controller setVideoPlayer: player];

    [[self stack] swapController: controller];
    [controller release];
	 */
	
}

@end
